# pc/main.py
from flask import Flask, render_template
from flask_socketio import SocketIO
from audio.audio_processor import PCAudioProcessor
#from camera.camera_receiver import CameraReceiver
#from camera.person_detector import PersonDetector
#from camera.video_recorder import VideoRecorder
import threading
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

# Flaskアプリケーションの設定
app = Flask(__name__)
socketio = SocketIO(app, cors_allowed_origins="*")

# コンポーネントの初期化
audio_processor = PCAudioProcessor()
#camera_receiver = CameraReceiver(12345)
#person_detector = PersonDetector('models/deploy.prototxt', 'models/mobilenet_iter_73000.caffemodel')
#video_recorder = VideoRecorder()

#@app.route('/')
#def index():
#    """メインページの表示"""
#    return render_template('index.html')

#@app.route('/camera')
#def camera():
#    """カメラページの表示"""
#    return render_template('camera.html')

@app.route('/')
def audio():
    """音声ページの表示"""
    return render_template('audio.html')

@socketio.on('connect')
def handle_connect():
    logger.info("Client connected")
    """socketio.emit('pc_status_update', {
        'mic_enabled': audio_processor.state.mic_enabled,
        'speaker_enabled': audio_processor.state.speaker_enabled,
        'volume': audio_processor.state.volume
    })"""

@socketio.on('pc_mic_toggle')
def handle_pc_mic_toggle(data):
    audio_processor.set_mic_enabled(data['enabled'])
    #socketio.emit('pc_status_update', {'mic_enabled': audio_processor.state.mic_enabled})

@socketio.on('pc_speaker_toggle')
def handle_pc_speaker_toggle(data):
    audio_processor.set_speaker_enabled(data['enabled'])
    #socketio.emit('pc_status_update', {'speaker_enabled': audio_processor.state.speaker_enabled})

@socketio.on('pc_volume_change')
def handle_pc_volume_change(data):
    audio_processor.set_volume(data['volume'])
    #socketio.emit('pc_status_update', {'volume': audio_processor.state.volume})

@socketio.on('audio_data')
def handle_audio_data(data):
    try:
        audio_data = bytes.fromhex(data['audio'])
        audio_processor.process_incoming_audio(audio_data)
    except Exception as e:
        logger.error(f"Error processing audio data: {e}")
        socketio.emit('error_message', {'message': 'Audio processing error'})

# Raspberry Pi制御用のイベント転送
@socketio.on('raspi_mic_toggle')
def handle_raspi_mic_toggle(data):
    socketio.emit('raspi_mic_toggle', data)

@socketio.on('raspi_speaker_toggle')
def handle_raspi_speaker_toggle(data):
    socketio.emit('raspi_speaker_toggle', data)

@socketio.on('raspi_volume_change')
def handle_raspi_volume_change(data):
    socketio.emit('raspi_volume_change', data)

@socketio.on('speak_text')
def handle_speak_text(data):
    socketio.emit('speak_text', data)
    logger.info(f"OK")

def send_audio():
    """音声データの送信ループ"""
    while True:
        if audio_processor.state.mic_enabled:
            audio_data = audio_processor.get_outgoing_audio()
            if audio_data:
                try:
                    socketio.emit('audio_data', {'audio': audio_data.hex()})
                except Exception as e:
                    logger.error(f"Error sending audio data: {e}")
        socketio.sleep(0.01)

#def start_camera_thread():
#    """カメラ受信スレッドの開始"""
#    try:
#        camera_receiver.start_receiving()
#   except Exception as e:
#        logger.error(f"Camera receiver error: {e}")

if __name__ == '__main__':
    try:
        # コンポーネントの初期化
        audio_processor.initialize()
        
        # カメラ受信スレッドの開始
        #camera_thread = threading.Thread(target=start_camera_thread)
        #camera_thread.daemon = True
        #camera_thread.start()

        # 音声送信タスクの開始
        socketio.start_background_task(send_audio)
        
        # Flaskアプリケーションの起動
        socketio.run(app, host='0.0.0.0', port=7000, debug=False)
        
    except Exception as e:
        logger.error(f"Application error: {e}")
    finally:
        audio_processor.cleanup()
        #camera_receiver.close()
        #video_recorder.cleanup()