// showTab関数（ホーム）
function showTab(tabId) {
    const tabs = document.querySelectorAll('.tab');
    const headers = document.querySelectorAll('.tab-header');

    tabs.forEach(tab => {
        tab.classList.remove('active');
    });
    headers.forEach(header => {
        header.classList.remove('active');
    });

    document.getElementById(tabId).classList.add('active');
    document.querySelector(`.tab-header[data-tab="${tabId}"]`).classList.add('active');
}

// WebSocket接続設定
let socket;

function connectWebSocket() {
    socket = io.connect('http://172.25.15.52:7000');
    //socket2 = io.connect('http://172.25.17.100:7000');

    // 接続が確立された時の処理
    socket.on('connect', () => {
        console.log("WebSocket接続が確立されました。");
    });

    // カメラ1のフレーム受信
    socket.on('camera_stream_home_1', (data) => {
        const imgElement = document.getElementById('cameraStream1');
        if (imgElement) {
            imgElement.src = `data:image/jpeg;base64,${data.image}`;
        }
    });

    // カメラ2のフレーム受信
    socket.on('camera_stream_home_2', (data) => {
        const imgElement = document.getElementById('cameraStream2');
        if (imgElement) {
            imgElement.src = `data:image/jpeg;base64,${data.image}`;
        }
    });

    // WebSocket接続が切れたときの処理 (再接続)
    socket.on('disconnect', () => {
        console.warn("WebSocket接続が切断されました。再接続を試みます...");
        setTimeout(connectWebSocket, 2000); // 2秒後に再接続
    });

    socket.on('error_message', (data) => {
        updateStatus(`エラー: ${data.message}`);
    });
/*
    socket.on('pc_status_update', (data) => {
        if ('mic_enabled' in data) {
            pcMicToggle.checked = data.mic_enabled;
        }
        if ('speaker_enabled' in data) {
            pcSpeakerToggle.checked = data.speaker_enabled;
        }
        if ('volume' in data) {
            pcVolumeSlider.value = data.volume;
            pcVolumeValue.textContent = data.volume;
        }
    });  ////////////////////////

    socket.on('raspi_status_update', (data) => {
        if ('mic_enabled' in data) {
            raspiMicToggle.checked = data.mic_enabled;
        }
        if ('speaker_enabled' in data) {
            raspiSpeakerToggle.checked = data.speaker_enabled;
        }
        if ('volume' in data) {
            raspiVolumeSlider.value = data.volume;
            raspiVolumeValue.textContent = data.volume;
        }
    });  *///////////////////////
}

// 初回接続
connectWebSocket();  //これ必要？



//音声相互通信設定
function updateStatus(message) {
    const statusMessage = document.getElementById('statusMessage');
    statusMessage.textContent = message;
    setTimeout(() => {
        statusMessage.textContent = '準備完了';
    }, 3000);
}

function pc_mic_toggle(){
    socket.emit('pc_mic_toggle', { enabled: pcMicToggle.checked });  //ture or folaseをpc_mic_toggleとして送信
    updateStatus(`PCマイクを${pcMicToggle.checked ? '有効' : '無効'}にしました`);  //ture or folse
}

function pc_speaker_toggle(){
    socket.emit('pc_speaker_toggle', { enabled: pcSpeakerToggle.checked });
    updateStatus(`PCスピーカーを${pcSpeakerToggle.checked ? '有効' : '無効'}にしました`);
}

function pc_volume_slider(){
    const volume = document.getElementById("pcVolumeSlider");  //値を格納
    const pcVolumeValue = document.getElementById("pcVolumeValue")
    pcVolumeValue.textContent = volume.value;  //表示変更
    socket.emit('pc_volume_change', { volume: parseInt(volume.value) });
}

function raspi_mic_toggle(){
    socket.emit('raspi_mic_toggle', { enabled: raspiMicToggle.checked });
    updateStatus(`Raspberry Piマイクを${raspiMicToggle.checked ? '有効' : '無効'}にしました`);
}

function raspi_speaker_toggle(){
    socket.emit('raspi_speaker_toggle', { enabled: raspiSpeakerToggle.checked });
    updateStatus(`Raspberry Piスピーカーを${raspiSpeakerToggle.checked ? '有効' : '無効'}にしました`);
}

function raspi_volume_slider(){
    const volume = document.getElementById("raspiVolumeSlider");
    const raspiVolumeValue = document.getElementById("raspiVolumeValue");
    raspiVolumeValue.textContent = volume.value;
    socket.emit('raspi_volume_change', { volume: parseInt(volume.value) });
}

function speak_toggle(){
    const speakButton = document.getElementById('speakButton');
    const textToSpeak = document.getElementById('textToSpeak');
    const text = textToSpeak.value.trim();  //空白削除
    if (text) {
        socket.emit('speak_text', { text: text });  //speak_textとしてtexttデータを送信
        speakButton.disabled = true;  //連打禁止
        updateStatus('テキストを読み上げ中...');
        setTimeout(() => {
            speakButton.disabled = false;
        }, 2000);
    } else {
        updateStatus('テキストを入力してください');
    }
}
// 音声発信
    // "聞く"ボタンの表示/非表示制御
    /*function toggleListen() {
        const hiddenContent = document.getElementById("hiddenContent");
        hiddenContent.style.display = hiddenContent.style.display === "none" ? "block" : "none";
    }*/

// 設定