import subprocess
import os
import signal
import atexit
import paramiko
from threading import Thread
from flask import Flask, render_template, request
from werkzeug.serving import make_server
#from flask_socketio import SocketIO  #追加1/7

# 仮想環境のPythonパス
VENV_PYTHON =  r"C:\Users\平松 拓朗\AppData\Local\Programs\Python\Python312\python.exe" #r"C:\Users\平松 拓朗\OneDrive - 独立行政法人 国立高等専門学校機構\課題\mirs\myproject\venv\Scripts\python.exe"

# 実行するPythonスクリプトのパス
#WINDOWS_RUNNER_PATH = r"C:\平松 拓朗\OneDrive - 独立行政法人 国立高等専門学校機構\課題\mirs\myproject\家作業\OneDrive_2024-12-16\mirs (1)\ssh_test\windows_runnner.py"
#APP_PATH = r"C:\Users\平松 拓朗\OneDrive - 独立行政法人 国立高等専門学校機構\課題\mirs\myproject\家作業\OneDrive_2024-12-16\mirs (1)\video_v2"
#APP_SCRIPT = "app.py"
APP2_PATH = r"C:\Users\平松 拓朗\OneDrive - 独立行政法人 国立高等専門学校機構\課題\mirs\myproject\家作業\OneDrive_2024-12-16\mirs (1)\device_control\pc"
APP2_SCRIPT = "pc_main.py"

# Raspberry Pi の接続情報
RASPI_HOST = "172.25.16.204"
RASPI_PORT = 22
RASPI_USER = "pi"
RASPI_PASSWORD = "toyogipro"

# Flask用
FLASK_PORT = 9000
TEMPLATES_DIR = os.path.join(os.path.dirname(__file__), "templates")  #現在のパスとtemplatesを結合してパスを得るC:\Users\平松 拓朗\OneDrive - 独立行政法人 国立高等専門学校機構\課題\mirs\myproject\家作業\OneDrive_2024-12-16\mirs (1)\templates

# 子プロセス管理
processes = []
#ssh_client = None
#raspi_pid = None
ssh_client2 = None
raspi_pid2 = None
flask_server = None

# Flask アプリケーションの設定
flask_app = Flask(__name__, template_folder=TEMPLATES_DIR)

@flask_app.route("/")
def home():
    return render_template("home.html")


def start_flask_server():
    """
    Flaskサーバーを起動し、home.htmlをport 9000で実行する
    """
    global flask_server
    print(f"Flaskサーバーを起動します (port: {FLASK_PORT}) ...")
    flask_server = make_server("0.0.0.0", FLASK_PORT, flask_app)
    flask_server.serve_forever()


def stop_flask_server():
    """
    Flaskサーバーを停止する
    """
    global flask_server
    if flask_server:
        print("Flaskサーバーを停止します...")
        flask_server.shutdown()
        print("Flaskサーバーが停止しました。")


def cleanup_processes():
    """
    子プロセスおよび Raspberry Pi 上のプロセスを強制終了する
    """
    global ssh_client2, raspi_pid2
    print("クリーンアップを行います...")

    # Flaskサーバーを停止
    stop_flask_server()

    # 子プロセスを強制終了
    for process in processes:
        try:
            if process.poll() is None:  # プロセスがまだ実行中の場合
                print(f"子プロセス (PID: {process.pid}) を強制終了します...")
                os.kill(process.pid, signal.SIGTERM)
        except Exception as e:
            print(f"プロセス終了時にエラーが発生しました: {e}")

    # Raspberry Pi 上のプロセスを強制終了
    if ssh_client2 and raspi_pid2:
        try:
            print(f"Raspberry Pi 上のプロセス (PID: {raspi_pid2}) を強制終了します...")
            ssh_client2.exec_command(f"kill -9 {raspi_pid2}")
            print("Raspberry Pi 上のプロセスを終了しました。")
        except Exception as e:
            print(f"Raspberry Pi プロセス強制終了時にエラーが発生しました: {e}")
        finally:
            ssh_client2.close()
            print("SSH接続を閉じました。")


def get_remote_pid():
    """
    Raspberry Pi 上の camera_control.py の PID を取得
    """
    global ssh_client2
    stdin, stdout, stderr = ssh_client2.exec_command("pgrep -f raspi_main.py")
    pid = stdout.read().decode().strip()
    return pid


def run_script(script_path, script_name):
    """
    指定したスクリプトを仮想環境のPythonで新しいプロセスとして実行
    """
    print(f"{script_name} を仮想環境で起動します...")
    process = subprocess.Popen(
        [VENV_PYTHON, os.path.join(script_path, script_name)],
        creationflags=subprocess.CREATE_NEW_PROCESS_GROUP,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
    )
    processes.append(process)
    return process

'''
def start_camera_control():
    """
    Raspberry Pi 上で camera_control.py を起動
    """
    global ssh_client, raspi_pid
    ssh_client = paramiko.SSHClient()
    ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh_client.connect(RASPI_HOST, port=RASPI_PORT, username=RASPI_USER, password=RASPI_PASSWORD)

    ssh_client.exec_command("nohup python3 /home/pi/test/myproject/raspi/camera_control.py > /tmp/camera_control.log 2>&1 &")
    raspi_pid = get_remote_pid()
    print(f"camera_control.py を起動しました。リモート PID: {raspi_pid}")
'''
def start_audio_control():
    '''
    Raspberry Pi 上で raspi_audio_main.py を起動
    '''
    global ssh_client2, raspi_pid2
    ssh_client2 = paramiko.SSHClient()
    ssh_client2.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh_client2.connect(RASPI_HOST, port=RASPI_PORT, username=RASPI_USER, password=RASPI_PASSWORD)
    ssh_client2.exec_command("nohup python3 /home/pi/mirs_taku/raspi9/raspi_main.py > /tmp/raspi_main.log 2>&1 &")
    raspi_pid2 = get_remote_pid()
    print(f"raspi_audio_main.py を起動しました。リモート PID: {raspi_pid2}")




def main():
    """
    メイン関数: 仮想環境のPythonを使って app.py と windows_runner.py を並列で実行
    """
    try:
        #start_camera_control()

        # Flaskサーバーを別スレッドで起動
        flask_thread = Thread(target=start_flask_server)
        flask_thread.start()

        # app.py と windows_runner.py を実行
        #app_process = run_script(APP_PATH, APP_SCRIPT)
        #runner_process = run_script(os.path.dirname(WINDOWS_RUNNER_PATH), os.path.basename(WINDOWS_RUNNER_PATH))
        app2_process = run_script(APP2_PATH, APP2_SCRIPT)
        #socketio = SocketIO(flask_app, cors_allowed_origins="*")  #追加1/7

        start_audio_control()

        print("すべてのプロセスが起動しました。Ctrl+C で終了します。")

        # 子プロセスの終了を待機
        while True:
            if app2_process.poll() is not None:
                print("app.py が終了しました。")
                break

    except KeyboardInterrupt:
        print("\nCtrl+C が検出されました。")
    finally:
        cleanup_processes()


if __name__ == "__main__":
    atexit.register(cleanup_processes)
    main()