#include <Servo.h>

Servo servo1, servo2, servo3, servo4;
String inputString = "";
boolean stringComplete = false;

// 初期角度を定義
const int INITIAL_ANGLES[4] = {25, 17, 137, 120};

// 可動範囲を定義
const int SERVO_LIMITS[4][2] = {
    {0, 50},    // サーボ1: 25°±25°
    {2, 32},    // サーボ2: 17°±15°
    {112, 162}, // サーボ3: 137°±25°
    {105, 135}  // サーボ4: 120°±15°
};

void setup() {
    Serial.begin(9600);
    
    servo1.attach(3);
    servo2.attach(5);
    servo3.attach(6);
    servo4.attach(10);
    
    inputString.reserve(200);
    
    // 初期位置に設定
    servo1.write(INITIAL_ANGLES[0]);
    servo2.write(INITIAL_ANGLES[1]);
    servo3.write(INITIAL_ANGLES[2]);
    servo4.write(INITIAL_ANGLES[3]);
}

void loop() {
    if (stringComplete) {
        int angles[4];
        angles[0] = getValue(inputString, ',', 0).toInt();
        angles[1] = getValue(inputString, ',', 1).toInt();
        angles[2] = getValue(inputString, ',', 2).toInt();
        angles[3] = getValue(inputString, ',', 3).toInt();
        
        // サーボの角度を設定
        servo1.write(constrain(angles[0], SERVO_LIMITS[0][0], SERVO_LIMITS[0][1]));
        servo2.write(constrain(angles[1], SERVO_LIMITS[1][0], SERVO_LIMITS[1][1]));
        servo3.write(constrain(angles[2], SERVO_LIMITS[2][0], SERVO_LIMITS[2][1]));
        servo4.write(constrain(angles[3], SERVO_LIMITS[3][0], SERVO_LIMITS[3][1]));

        // 設定された角度を出力
        Serial.print(constrain(angles[0], SERVO_LIMITS[0][0], SERVO_LIMITS[0][1]));
        Serial.print(",");
        Serial.print(constrain(angles[1], SERVO_LIMITS[1][0], SERVO_LIMITS[1][1]));
        Serial.print(",");
        Serial.print(constrain(angles[2], SERVO_LIMITS[2][0], SERVO_LIMITS[2][1]));
        Serial.print(",");
        Serial.println(constrain(angles[3], SERVO_LIMITS[3][0], SERVO_LIMITS[3][1]));
        
        inputString = "";
        stringComplete = false;
    }
}

void serialEvent() {
    while (Serial.available()) {
        char inChar = (char)Serial.read();
        if (inChar == '\n') {
            stringComplete = true;
        } else {
            inputString += inChar;
        }
    }
}

String getValue(String data, char separator, int index) {
    int found = 0;
    int strIndex[] = {0, -1};
    int maxIndex = data.length() - 1;

    for (int i = 0; i <= maxIndex && found <= index; i++) {
        if (data.charAt(i) == separator || i == maxIndex) {
            found++;
            strIndex[0] = strIndex[1] + 1;
            strIndex[1] = (i == maxIndex) ? i + 1 : i;
        }
    }
    return found > index ? data.substring(strIndex[0], strIndex[1]) : "";
}