// encoder.ino

// エンコーダの状態変数をvolatileとして定義
static volatile int8_t last_a_l = 0;
static volatile int8_t last_b_l = 0;
static volatile int8_t last_a_r = 0;
static volatile int8_t last_b_r = 0;

// デバウンス用の変数を追加
static volatile unsigned long last_interrupt_time_l = 0;
static volatile unsigned long last_interrupt_time_r = 0;
const unsigned long DEBOUNCE_TIME = 30; // マイクロ秒単位

static void enc_change_l() {
    // デバウンス処理
    unsigned long interrupt_time = micros();
    if (interrupt_time - last_interrupt_time_l < DEBOUNCE_TIME) {
        return;
    }
    last_interrupt_time_l = interrupt_time;
    
    int8_t a_curr = digitalRead(PIN_ENC_A_L);
    int8_t b_curr = digitalRead(PIN_ENC_B_L);
    
    // Gray codeによる方向検出
    int8_t dir = 0;
    if (a_curr != last_a_l) {  // Aピンの変化
        dir = (a_curr == b_curr) ? 1 : -1;
    }
    if (b_curr != last_b_l) {  // Bピンの変化
        dir = (a_curr == b_curr) ? -1 : 1;
    }
    
    count_l += dir;
    last_a_l = a_curr;
    last_b_l = b_curr;
}

static void enc_change_r() {
    // デバウンス処理
    unsigned long interrupt_time = micros();
    if (interrupt_time - last_interrupt_time_r < DEBOUNCE_TIME) {
        return;
    }
    last_interrupt_time_r = interrupt_time;
    
    int8_t a_curr = digitalRead(PIN_ENC_A_R);
    int8_t b_curr = digitalRead(PIN_ENC_B_R);
    
    // Gray codeによる方向検出
    int8_t dir = 0;
    if (a_curr != last_a_r) {  // Aピンの変化
        dir = (a_curr == b_curr) ? 1 : -1;
    }
    if (b_curr != last_b_r) {  // Bピンの変化
        dir = (a_curr == b_curr) ? -1 : 1;
    }
    
    count_r += dir;
    last_a_r = a_curr;
    last_b_r = b_curr;
}

void encoder_open() {
    // プルアップ抵抗を有効化してノイズ耐性を向上
    pinMode(PIN_ENC_A_L, INPUT_PULLUP);
    pinMode(PIN_ENC_B_L, INPUT_PULLUP);
    pinMode(PIN_ENC_A_R, INPUT_PULLUP);
    pinMode(PIN_ENC_B_R, INPUT_PULLUP);
    
    // 初期状態の読み取り
    last_a_l = digitalRead(PIN_ENC_A_L);
    last_b_l = digitalRead(PIN_ENC_B_L);
    last_a_r = digitalRead(PIN_ENC_A_R);
    last_b_r = digitalRead(PIN_ENC_B_R);
    
    last_interrupt_time_l = micros();
    last_interrupt_time_r = micros();
    
    // 両方のピンに割り込みを設定
    attachInterrupt(digitalPinToInterrupt(PIN_ENC_A_L), enc_change_l, CHANGE);
    attachInterrupt(digitalPinToInterrupt(PIN_ENC_B_L), enc_change_l, CHANGE);
    attachInterrupt(digitalPinToInterrupt(PIN_ENC_A_R), enc_change_r, CHANGE);
    attachInterrupt(digitalPinToInterrupt(PIN_ENC_B_R), enc_change_r, CHANGE);
}

void encoder_reset() {
    noInterrupts();
    count_l = 0;
    count_r = 0;
    last_a_l = digitalRead(PIN_ENC_A_L);
    last_b_l = digitalRead(PIN_ENC_B_L);
    last_a_r = digitalRead(PIN_ENC_A_R);
    last_b_r = digitalRead(PIN_ENC_B_R);
    last_interrupt_time_l = micros();
    last_interrupt_time_r = micros();
    interrupts();
}

void encoder_get(int *cnt_l, int *cnt_r) {
    noInterrupts();
    *cnt_l = count_l;
    *cnt_r = count_r;
    interrupts();
}
