// types.h
#ifndef TYPES_H
#define TYPES_H

// オドメトリデータ構造体
struct OdometryData {
    float x;          // X位置 [m]
    float y;          // Y位置 [m]
    float theta;      // 姿勢角 [rad]
    float velocity;   // 並進速度 [m/s]
    float omega;      // 角速度 [rad/s]
};

// 速度制御用の構造体
struct VelocityCommand {
    double linear_x;   // 並進速度 [m/s]
    double angular_z;  // 角速度 [rad/s]
};

// モータパラメータ構造体
struct MotorParameters {
    double kp;               // 比例ゲイン
    double ki;               // 積分ゲイン
    double kd;               // 微分ゲイン
    double min_power;        // 最小出力
    double max_power;        // 最大出力
    double power_scale;      // 出力スケール係数
    bool reverse_direction;  // 方向反転フラグ
};

// モータ制御クラス
class MotorController {
private:
    double target;              // 目標値
    double previous_error;      // 前回の偏差
    double integral;            // 積分値
    double filtered_derivative; // フィルタ済み微分値
    double last_output;         // 前回の出力値
    double last_velocity;       // 前回の速度
    bool is_rotation_mode;      // 回転モードフラグ
    MotorParameters params;     // モータパラメータ
    
    // 回転モード用の状態変数
    struct {
        double base_power;   // 基準出力
        double last_output;  // 前回の出力
    } rotation_state;

public:
    MotorController(const MotorParameters& parameters);
    void setTarget(double new_target);
    void setRotationMode(bool rotation_mode);
    void reset();
    double compute(double current_value, double dt);

private:
    double computeLinear(double current_value, double dt);
    double computeRotation(double current_value, double dt);
};

// グローバル変数の宣言
extern volatile int count_l;
extern volatile int count_r;
extern int last_count_l;
extern int last_count_r;
extern VelocityCommand vel_cmd;
extern OdometryData odom_data;
extern MotorController left_motor;
extern MotorController right_motor;
extern double wheel_velocity_left;
extern double wheel_velocity_right;

#endif // TYPES_H
