# === robot_control/launch/test_navigation.launch.py ===
from launch import LaunchDescription
from launch.actions import IncludeLaunchDescription, DeclareLaunchArgument
from launch.launch_description_sources import PythonLaunchDescriptionSource
from launch.substitutions import LaunchConfiguration, PathJoinSubstitution
from launch_ros.actions import Node
from launch_ros.substitutions import FindPackageShare
from ament_index_python.packages import get_package_share_directory
import os

def generate_launch_description():
    # パッケージのパスを取得
    robot_control_pkg = get_package_share_directory('robot_control')
    map_pkg = get_package_share_directory('my_school_map')
    
    # 設定ファイルのパス
    nav_params_path = os.path.join(robot_control_pkg, 'config', 'navigation_params.yaml')
    map_path = os.path.join(map_pkg, 'maps', 'school_map.yaml')
    
    # パラメータ設定
    use_sim_time = LaunchConfiguration('use_sim_time', default='false')
    
    return LaunchDescription([
        # パラメータ宣言
        DeclareLaunchArgument(
            'use_sim_time',
            default_value='false',
            description='Use simulation clock'
        ),
            
        # マップサーバーの起動
        IncludeLaunchDescription(
            PythonLaunchDescriptionSource([
                os.path.join(map_pkg, 'launch', 'map_install_launch.py')
            ])
        ),
        
        # ロボット制御ノードの起動
        Node(
            package='robot_control',
            executable='robot_navigation_control',
            name='robot_navigation_control',
            parameters=[nav_params_path],
            output='screen'
        ),
        
        # SLLiDARノードの起動（パラメータを修正）
        Node(
            package='sllidar_ros2',
            executable='sllidar_node',
            name='sllidar_node',
            parameters=[{
                'serial_port': '/dev/ttyUSB0',
                'serial_baudrate': 256000,
                'frame_id': 'laser',
                'angle_compensate': True,
                'scan_mode': 'DenseBoost',  # より高密度なスキャンモード
                'sample_rate': 9,           # 9kHzのサンプリングレート
                'min_distance': 0.40,       # 最小検出距離を45cmに設定
                'max_distance': 8.0,        # 最大検出距離を8mに制限
                'range_min': 0.35,          # 有効な最小距離
                'range_max': 10.0,           # 有効な最大距離
                'angle_min': 0.78539,      # スキャン開始角度
                'angle_max': 2.35619,       # スキャン終了角度
                'skip_samples': '2'         # サンプルのスキップ数
            }],
            output='screen'
        ),
        
        # TF関連ノードの起動
        Node(
            package='tf2_ros',
            executable='static_transform_publisher',
            name='base_to_laser_broadcaster',
            arguments=['-0.115', '0.02', '0.3', '1.57', '0', '0', 'base_link', 'laser']
        )
    ])
