# raspi/main.py
import socketio
import threading
import logging
#import cv2
from audio.audio_handler import RaspberryPiAudio
#from camera.camera_handler import CameraHandler
#from camera.camera_sender import CameraSender

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

server_url = "http://172.25.15.52:7000"
sio = socketio.Client()
audio_handler = RaspberryPiAudio()  #イニシャライズ化


@sio.event
def connect():
    logger.info("Connected to server")
    #send_status_update()

@sio.event
def disconnect():
    logger.info("Disconnected from server")

@sio.on('raspi_mic_toggle')
def handle_raspi_mic_toggle(data):
    audio_handler.set_mic_enabled(data['enabled'])
    #send_status_update()

@sio.on('raspi_speaker_toggle')
def handle_raspi_speaker_toggle(data):
    audio_handler.set_speaker_enabled(data['enabled'])
    #send_status_update()

@sio.on('raspi_volume_change')
def handle_raspi_volume_change(data):
    audio_handler.set_volume(data['volume'])
    #send_status_update()

@sio.on('speak_text')
def handle_speak_text(data):
    text = data.get('text')
    logger.info(f"Text input OK")
    if text:
        audio_handler.text_queue.put(text)
        #send_status_update(status='speaking')

@sio.on('audio_data')
def handle_audio_data(data):
    try:
        audio_data = bytes.fromhex(data['audio'])
        audio_handler.process_incoming_audio(audio_data)
    except Exception as e:
        logger.error(f"Error processing audio data: {e}")
"""
def send_status_update(status=None):
    try:
        update_data = {
            'mic_enabled': audio_handler.state.mic_enabled,
            'speaker_enabled': audio_handler.state.speaker_enabled,
            'volume': audio_handler.state.volume
        }
        if status:
            update_data['status'] = status
        sio.emit('raspi_status_update', update_data)
    except Exception as e:
        logger.error(f"Failed to send status update: {e}")
"""
def send_audio_loop():
    while True:
        if audio_handler.state.mic_enabled:
            audio_data = audio_handler.get_outgoing_audio()
            if audio_data:
                try:
                    sio.emit('audio_data', {'audio': audio_data.hex()})
                except Exception as e:
                    logger.error(f"Error sending audio data: {e}")
        threading.Event().wait(0.01)

def start():
    try:
        # 音声システムの初期化
        audio_handler.initialize()
        
        # サーバーへの接続
        sio.connect(server_url)
        
        # オーディオ送信スレッドの開始
        threading.Thread(target=send_audio_loop, daemon=True).start()
        
        # メインスレッドを維持
        sio.wait()  #非同期通信を維持
        
    except KeyboardInterrupt:
        logger.info("Shutting down...")
    except Exception as e:
        logger.error(f"Error: {e}")
    finally:
        cleanup()

def cleanup():
    if sio.connected:
        sio.disconnect()

    audio_handler.cleanup()

def main():
    try:
        audio_thread = threading.Thread(target=start)  #start関数を別スレッドで実行
        audio_thread.start()

        # カメラの初期化と開始
        #camera_handler = CameraHandler()
        #camera_handler.initialize()
        
        ##camera_sender = CameraSender('localhost', 12345)
        #camera_thread = threading.Thread(target=lambda: camera_sender.start(camera_handler))
        #camera_thread.start()

        # スレッドの終了を待機
        audio_thread.join()
        #camera_thread.join()

    except KeyboardInterrupt:
        logger.info("Shutting down...")
    except Exception as e:
        logger.error(f"Application error: {e}")
    #finally:
        #camera_handler.cleanup()

if __name__ == "__main__":
    main()
