import subprocess
import time
import os
import signal

class DualCameraStreamer:
    def __init__(self):
        self.processes = []
        self.running = True

    def start_streaming(self):
        try:
            # 各カメラの設定を調整
            cmd1 = [
                'mjpg_streamer',
                '-i',
                'input_uvc.so -d /dev/video0 -r 640x480 -f 10',  # フレームレートを10に変更
                '-o',
                'output_http.so -p 8080'
            ]
            
            cmd2 = [
                'mjpg_streamer',
                '-i',
                'input_uvc.so -d /dev/video2 -r 640x480 -f 10',  # フレームレートを10に変更
                '-o',
                'output_http.so -p 8081'
            ]

            # 一つずつプロセスを開始
            self.processes.append(subprocess.Popen(cmd1))
            print("Camera 1 started")
            time.sleep(2)  # 十分な待機時間
            
            self.processes.append(subprocess.Popen(cmd2))
            print("Camera 2 started")

            print("Access streams at:")
            print("Camera 1: http://172.25.16.204:8080/?action=stream")
            print("Camera 2: http://172.25.16.204:8081/?action=stream")

            # 監視ループ
            while self.running:
                time.sleep(1)

        except Exception as e:
            print(f"Error: {e}")
            self.stop_streaming()

    def stop_streaming(self):
        print("Stopping streams...")
        for process in self.processes:
            try:
                process.terminate()
                process.wait(timeout=3)
            except:
                process.kill()
        self.processes = []

def main():
    streamer = DualCameraStreamer()
    try:
        streamer.start_streaming()
    except KeyboardInterrupt:
        print("\nStopping...")
    finally:
        streamer.running = False
        streamer.stop_streaming()

if __name__ == "__main__":
    main()